@@ -88,7 +88,7 @@ pub enum PartialToken {
     VerticalBar,
 }
 
-// Make this a const fn as soon as match gets stable (issue #57563)
+// Make this a const fn as soon as is_whitespace and to_string get stable (issue #57563)
 fn char_to_partial_token(c: char) -> PartialToken {
     match c {
         '+' => PartialToken::Plus,
@@ -122,8 +122,8 @@ fn char_to_partial_token(c: char) -> PartialToken {
 }
 
 impl Token {
-    // Make this a const fn as soon as match gets stable (issue #57563)
-    pub(crate) fn is_leftsided_value(&self) -> bool {
+    #[cfg(not(tarpaulin_include))]
+    pub(crate) const fn is_leftsided_value(&self) -> bool {
         match self {
             Token::Plus => false,
             Token::Minus => false,
@@ -166,8 +166,8 @@ impl Token {
         }
     }
 
-    // Make this a const fn as soon as match gets stable (issue #57563)
-    pub(crate) fn is_rightsided_value(&self) -> bool {
+    #[cfg(not(tarpaulin_include))]
+    pub(crate) const fn is_rightsided_value(&self) -> bool {
         match self {
             Token::Plus => false,
             Token::Minus => false,
@@ -210,6 +210,7 @@ impl Token {
         }
     }
 
+    #[cfg(not(tarpaulin_include))]
     pub(crate) fn is_assignment(&self) -> bool {
         use Token::*;
         matches!(
@@ -354,7 +355,26 @@ fn partial_tokens_to_tokens(mut tokens: &[PartialToken]) -> EvalexprResult<Vec<T
                     } else if let Ok(boolean) = literal.parse::<bool>() {
                         Some(Token::Boolean(boolean))
                     } else {
-                        Some(Token::Identifier(literal.to_string()))
+                        // If there are two tokens following this one, check if the next one is
+                        // a plus or a minus. If so, then attempt to parse all three tokens as a
+                        // scientific notation number of the form `<coefficient>e{+,-}<exponent>`,
+                        // for example [Literal("10e"), Minus, Literal("3")] => "1e-3".parse().
+                        match (second, third) {
+                            (Some(second), Some(third))
+                                if second == PartialToken::Minus
+                                    || second == PartialToken::Plus =>
+                            {
+                                if let Ok(number) =
+                                    format!("{}{}{}", literal, second, third).parse::<FloatType>()
+                                {
+                                    cutoff = 3;
+                                    Some(Token::Float(number))
+                                } else {
+                                    Some(Token::Identifier(literal.to_string()))
+                                }
+                            }
+                            _ => Some(Token::Identifier(literal.to_string())),
+                        }
                     }
                 },
                 PartialToken::Whitespace => {
@@ -421,3 +441,36 @@ fn partial_tokens_to_tokens(mut tokens: &[PartialToken]) -> EvalexprResult<Vec<T
 pub(crate) fn tokenize(string: &str) -> EvalexprResult<Vec<Token>> {
     partial_tokens_to_tokens(&str_to_partial_tokens(string)?)
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::token::{char_to_partial_token, tokenize};
+
+    #[test]
+    fn test_partial_token_display() {
+        let chars = vec![
+            '+', '-', '*', '/', '%', '^', '(', ')', ',', ';', '=', '!', '>', '<', '&', '|', ' ',
+        ];
+
+        for char in chars {
+            assert_eq!(
+                format!("{}", char),
+                format!("{}", char_to_partial_token(char))
+            );
+        }
+    }
+
+    #[test]
+    fn test_token_display() {
+        let token_string =
+            "+ - * / % ^ == != > < >= <= && || ! ( ) = += -= *= /= %= ^= &&= ||= , ; ";
+        let tokens = tokenize(token_string).unwrap();
+        let mut result_string = String::new();
+
+        for token in tokens {
+            result_string += &format!("{} ", token);
+        }
+
+        assert_eq!(token_string, result_string);
+    }
+}
